##################################################################
# Created by    : Adamson dela Cruz
# Date Created  : 8/31/17
# Remarks       : The commandline interface for creating things in
#                 AWS IoT
##################################################################

from __future__ import print_function
from factories.aws.thingtype import AWSThingType
from factories.aws.thing import AWSThing
import os, json
import click

@click.group()
def main():
    '''
    Entry point of the script
    '''
    pass

def check_env():
    '''
    Checks environment variables if set. If not, do not proceed
    '''
    #
    if not 'AWS_PROFILE' in os.environ:
        print('AWS_PROFILE is not set. To set the environment run the command below.\nexport AWS_PROFILE=MYAWSPROFILE\n')
        exit(1)

    if not 'AWS_REGION' in os.environ:
        print('AWS_REGION is not set. To set the environment run the command below.\nexport AWS_REGION=region-name\n')
        exit(1)



@click.option('-d','--description', default='', help='Describes the type')
@click.option('-a', 'attributes', help='Json attribute of the type.')
@click.argument('name')
@main.command()
def newthingtype(name, provider, attributes, description):
    check_env()

    fields = attributes.split(',')
    payload = {
        "thingTypeDescription": description,
        "searchableAttributes": fields
    }

    result = AWSThingType(name, payload)

    return result

@click.option('-t','--type', help='Derive from this type')
@click.option('-a', 'attributes', help='Json attribute file for the thing. Maximum of 3 attributes e.g. name,color,temperature')
@click.argument('name')
@main.command()
def newthing(name, provider, attributes, type):

    check_env()

    payload = None
    with open(attributes) as json_data:
        payload = json.load(json_data)

    thing = AWSThing()
    result = thing.new_thing(name, type, payload)

    return result


@click.option('-a', 'attributes', help='Json attribute file for the thing. Maximum of 3 attributes e.g. name,color,temperature')
@click.argument('name')
@main.command()
def updatething(name, provider, attributes):

    check_env()

    payload = None
    with open(attributes) as json_data:
        payload = json.load(json_data)

    thing = AWSThing()
    result = thing.update_thing(name, None, payload)

    return result

@click.argument('certname')
@click.argument('name')
@main.command()
def attachcert(name, certname):

    check_env()

    thing = AWSThing()
    result = thing.attach_certificate(name, certname)

    return result


##################################################################
# Created by    : Adamson dela Cruz
# Date Created  : 8/31/17
# Remarks       : The AWS Thing
##################################################################

from __future__ import print_function
from factories.aws import __client, initialize

client = __client

class AWSThing:
    ''' The device representation as a thing in AWS IoT '''

    ##################################################################
    #  Initializes the AWS client object
    ##################################################################
    def init_client(self):

        global client
        try:
            if client is None:
                client = initialize()
                if not type(client).__name__ == 'IoT':
                    print('The client variable resolve into an invalid type... ')
                    exit(1)

        except Exception as ex:
            print("ERROR:", ex)


    ##################################################################
    # Creates a new thing from the thing
    ##################################################################
    def new_thing(self, name, thingtype, payload):

        global client
        self.init_client()

        try:
            print("Creating new thing ", name)

            if thingtype is None:
                response = client.create_thing(
                    thingName = name,
                    attributePayload={
                        'attributes': payload,
                        'merge': False # Overwrite the property in the registry
                    }

                )
                print(response)

            else:
                response = client.create_thing(
                    thingName=name,
                    thingTypeName=thingtype,
                    attributePayload={
                        'attributes': payload,
                        'merge': False  # Overwrite the property in the registry
                    }

                )
                print(response)

        except Exception as ex:
            print("ERROR:", ex)


    ##################################################################
    # Updates a thing from the thing
    ##################################################################
    def update_thing(self, name, thingtype, payload):

        global client
        self.init_client()

        try:
            print("Updating thing ", name)

            if thingtype is None:
                response = client.update_thing(
                    thingName = name,
                    attributePayload={
                        'attributes': payload,
                        'merge': False # Overwrite the property in the registry
                    }

                )
                print(response)

            else:
                response = client.update_thing(
                    thingName=name,
                    thingTypeName=thingtype,
                    attributePayload={
                        'attributes': payload,
                        'merge': False  # Overwrite the property in the registry
                    }

                )
                print(response)

        except Exception as ex:
            print("ERROR:", ex)


    ##################################################################
    # Attaches a certificate to an existing thing
    ##################################################################
    def attach_certificate(self, name, cert_name):

        global client
        self.init_client()

        try:
            response = client.attach_thing_principal(
                thingName=name,
                principal=cert_name
            )

            print("CERTIFICATE:", response)

        except Exception as ex:
            print("ERROR:", ex)


import boto3, os
__client = None

def initialize():
    print('INITIALIZING AWS....')
    __client = boto3.client('iot', region_name=os.environ['AWS_REGION'])
    print(os.environ['AWS_REGION'])
    return  __client


##################################################################
# Created by    : Adamson dela Cruz
# Date Created  : 8/31/17
# Remarks       : AWS Thing Type
##################################################################

from __future__ import print_function
from factories.aws import __client, initialize

client = __client

class AWSThingType:
    ''' The device representation as a thing type in AWS IoT '''

    def __init__(self, name, props):
        ''' Creates a new thing type. Thing Types has a maximum of only 3 attributes '''
        global client
        try:

            if client is None:
                client = initialize()
                if not type(client).__name__ == 'IoT':
                    print('The client variable resolve into an invalid type... ')
                    exit(1)
                response = client.create_thing_type(
                    thingTypeName=name,
                    thingTypeProperties=props
                )
                print(response)

        except Exception as ex:
            print("ERROR:", ex)


from setuptools import setup, find_packages

setup(
    name='aws-iot',
    description='Command Line utility to easily provision IoT things in AWS',

    version='0.0.1',

    # Author details
    author='Adamson dela Cruz',
    author_email='adamson.delacruz@gmail.com',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Utilities',

        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='aws iot',

    #packages=['commands'],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    install_requires=[
        'click', 'boto3', 'botocore'
    ],
    entry_points={
        'console_scripts': [
            'aws-iot=commands.awsiot:main',
        ],
    },
)